@@ -1,7 +1,7 @@
 from unittest.mock import patch
 
 import pytest
-
+from redis import Redis
 from redis.commands.graph import Edge, Node, Path
 from redis.commands.graph.execution_plan import Operation
 from redis.commands.graph.query_result import (
@@ -20,23 +20,37 @@ from redis.commands.graph.query_result import (
     QueryResult,
 )
 from redis.exceptions import ResponseError
-from tests.conftest import skip_if_redis_enterprise
+from tests.conftest import _get_client, skip_if_redis_enterprise, skip_if_resp_version
 
 
 @pytest.fixture
-def client(modclient):
-    modclient.flushdb()
-    return modclient
+def client(request, stack_url):
+    r = _get_client(
+        Redis, request, decode_responses=True, from_url="redis://localhost:6480"
+    )
+    r.flushdb()
+    return r
 
 
-@pytest.mark.redismod
+@pytest.mark.graph
+@skip_if_resp_version(3)
 def test_bulk(client):
     with pytest.raises(NotImplementedError):
         client.graph().bulk()
         client.graph().bulk(foo="bar!")
 
 
-@pytest.mark.redismod
+@pytest.mark.graph
+def test_graph_creation_throws_deprecation_warning(client):
+    """Verify that a DeprecationWarning is raised when creating a Graph instance."""
+
+    match = "RedisGraph support is deprecated as of Redis Stack 7.2"
+    with pytest.warns(DeprecationWarning, match=match):
+        client.graph()
+
+
+@pytest.mark.graph
+@skip_if_resp_version(3)
 def test_graph_creation(client):
     graph = client.graph()
 
@@ -81,7 +95,8 @@ def test_graph_creation(client):
     graph.delete()
 
 
-@pytest.mark.redismod
+@pytest.mark.graph
+@skip_if_resp_version(3)
 def test_array_functions(client):
     query = """CREATE (p:person{name:'a',age:32, array:[0,1,2]})"""
     client.graph().query(query)
@@ -102,7 +117,8 @@ def test_array_functions(client):
     assert [a] == result.result_set[0][0]
 
 
-@pytest.mark.redismod
+@pytest.mark.graph
+@skip_if_resp_version(3)
 def test_path(client):
     node0 = Node(node_id=0, label="L1")
     node1 = Node(node_id=1, label="L1")
@@ -122,7 +138,8 @@ def test_path(client):
     assert expected_results == result.result_set
 
 
-@pytest.mark.redismod
+@pytest.mark.graph
+@skip_if_resp_version(3)
 def test_param(client):
     params = [1, 2.3, "str", True, False, None, [0, 1, 2], r"\" RETURN 1337 //"]
     query = "RETURN $param"
@@ -132,7 +149,8 @@ def test_param(client):
         assert expected_results == result.result_set
 
 
-@pytest.mark.redismod
+@pytest.mark.graph
+@skip_if_resp_version(3)
 def test_map(client):
     query = "RETURN {a:1, b:'str', c:NULL, d:[1,2,3], e:True, f:{x:1, y:2}}"
 
@@ -149,7 +167,8 @@ def test_map(client):
     assert actual == expected
 
 
-@pytest.mark.redismod
+@pytest.mark.graph
+@skip_if_resp_version(3)
 def test_point(client):
     query = "RETURN point({latitude: 32.070794860, longitude: 34.820751118})"
     expected_lat = 32.070794860
@@ -166,7 +185,8 @@ def test_point(client):
     assert abs(actual["longitude"] - expected_lon) < 0.001
 
 
-@pytest.mark.redismod
+@pytest.mark.graph
+@skip_if_resp_version(3)
 def test_index_response(client):
     result_set = client.graph().query("CREATE INDEX ON :person(age)")
     assert 1 == result_set.indices_created
@@ -181,7 +201,8 @@ def test_index_response(client):
         client.graph().query("DROP INDEX ON :person(age)")
 
 
-@pytest.mark.redismod
+@pytest.mark.graph
+@skip_if_resp_version(3)
 def test_stringify_query_result(client):
     graph = client.graph()
 
@@ -235,7 +256,8 @@ def test_stringify_query_result(client):
     graph.delete()
 
 
-@pytest.mark.redismod
+@pytest.mark.graph
+@skip_if_resp_version(3)
 def test_optional_match(client):
     # Build a graph of form (a)-[R]->(b)
     node0 = Node(node_id=0, label="L1", properties={"value": "a"})
@@ -260,7 +282,8 @@ def test_optional_match(client):
     graph.delete()
 
 
-@pytest.mark.redismod
+@pytest.mark.graph
+@skip_if_resp_version(3)
 def test_cached_execution(client):
     client.graph().query("CREATE ()")
 
@@ -278,7 +301,8 @@ def test_cached_execution(client):
     assert cached_result.cached_execution
 
 
-@pytest.mark.redismod
+@pytest.mark.graph
+@skip_if_resp_version(3)
 def test_slowlog(client):
     create_query = """CREATE (:Rider
     {name:'Valentino Rossi'})-[:rides]->(:Team {name:'Yamaha'}),
@@ -291,7 +315,8 @@ def test_slowlog(client):
     assert results[0][2] == create_query
 
 
-@pytest.mark.redismod
+@pytest.mark.graph
+@skip_if_resp_version(3)
 @pytest.mark.xfail(strict=False)
 def test_query_timeout(client):
     # Build a sample graph with 1000 nodes.
@@ -306,7 +331,8 @@ def test_query_timeout(client):
         assert False is False
 
 
-@pytest.mark.redismod
+@pytest.mark.graph
+@skip_if_resp_version(3)
 def test_read_only_query(client):
     with pytest.raises(Exception):
         # Issue a write query, specifying read-only true,
@@ -315,7 +341,8 @@ def test_read_only_query(client):
         assert False is False
 
 
-@pytest.mark.redismod
+@pytest.mark.graph
+@skip_if_resp_version(3)
 def test_profile(client):
     q = """UNWIND range(1, 3) AS x CREATE (p:Person {v:x})"""
     profile = client.graph().profile(q).result_set
@@ -330,7 +357,8 @@ def test_profile(client):
     assert "Node By Label Scan | (p:Person) | Records produced: 3" in profile
 
 
-@pytest.mark.redismod
+@pytest.mark.graph
+@skip_if_resp_version(3)
 @skip_if_redis_enterprise()
 def test_config(client):
     config_name = "RESULTSET_SIZE"
@@ -362,8 +390,9 @@ def test_config(client):
     client.graph().config("RESULTSET_SIZE", -100, set=True)
 
 
-@pytest.mark.redismod
 @pytest.mark.onlynoncluster
+@pytest.mark.graph
+@skip_if_resp_version(3)
 def test_list_keys(client):
     result = client.graph().list_keys()
     assert result == []
@@ -386,7 +415,8 @@ def test_list_keys(client):
     assert result == []
 
 
-@pytest.mark.redismod
+@pytest.mark.graph
+@skip_if_resp_version(3)
 def test_multi_label(client):
     redis_graph = client.graph("g")
 
@@ -412,7 +442,8 @@ def test_multi_label(client):
         assert True
 
 
-@pytest.mark.redismod
+@pytest.mark.graph
+@skip_if_resp_version(3)
 def test_cache_sync(client):
     pass
     return
@@ -422,7 +453,7 @@ def test_cache_sync(client):
     # Client B will try to get Client A out of sync by:
     # 1. deleting the graph
     # 2. reconstructing the graph in a different order, this will casuse
-    #    a differance in the current mapping between string IDs and the
+    #    a difference in the current mapping between string IDs and the
     #    mapping Client A is aware of
     #
     # Client A should pick up on the changes by comparing graph versions
@@ -485,7 +516,8 @@ def test_cache_sync(client):
     assert A._relationship_types[1] == "R"
 
 
-@pytest.mark.redismod
+@pytest.mark.graph
+@skip_if_resp_version(3)
 def test_execution_plan(client):
     redis_graph = client.graph("execution_plan")
     create_query = """CREATE
@@ -504,7 +536,8 @@ def test_execution_plan(client):
     redis_graph.delete()
 
 
-@pytest.mark.redismod
+@pytest.mark.graph
+@skip_if_resp_version(3)
 def test_explain(client):
     redis_graph = client.graph("execution_plan")
     # graph creation / population
@@ -593,7 +626,8 @@ Project
     redis_graph.delete()
 
 
-@pytest.mark.redismod
+@pytest.mark.graph
+@skip_if_resp_version(3)
 def test_resultset_statistics(client):
     with patch.object(target=QueryResult, attribute="_get_stat") as mock_get_stats:
         result = client.graph().query("RETURN 1")
