@@ -1,11 +1,9 @@
 import pytest
-
 from redis.commands.graph import edge, node
 
 
-@pytest.mark.redismod
+@pytest.mark.graph
 def test_init():
-
     with pytest.raises(AssertionError):
         edge.Edge(None, None, None)
         edge.Edge(node.Node(), None, None)
@@ -16,7 +14,7 @@ def test_init():
     )
 
 
-@pytest.mark.redismod
+@pytest.mark.graph
 def test_to_string():
     props_result = edge.Edge(
         node.Node(), None, node.Node(), properties={"a": "a", "b": 10}
@@ -29,7 +27,7 @@ def test_to_string():
     assert no_props_result == ""
 
 
-@pytest.mark.redismod
+@pytest.mark.graph
 def test_stringify():
     john = node.Node(
         alias="a",
@@ -62,8 +60,8 @@ def test_stringify():
     )
 
 
-@pytest.mark.redismod
-def test_comparision():
+@pytest.mark.graph
+def test_comparison():
     node1 = node.Node(node_id=1)
     node2 = node.Node(node_id=2)
     node3 = node.Node(node_id=3)
